﻿using System;
using System.Collections.Generic;
using System.Text;
using Oxide.Core.Database;
using Oxide.Core.Libraries.Covalence;
using UnityEngine;
using System.Collections;
using Oxide.Core;
using UnityEngine.Scripting;
using System.Linq;
using Oxide.Core.MySql;
using System.Text.RegularExpressions;
using Oxide.Core.Plugins;

namespace Oxide.Plugins
{
    [Info("PvpArena", "531devv (531devv@gmail.com)", "1.7.3")]
    [Description("Pvp arena event.")]

    class PvpArena : HurtworldPlugin
    {

        #region ClassRegion

        class ArenaData
        {
            public Dictionary<string, Arena> Arena = new Dictionary<string, Arena>();
            public Dictionary<string, ArenaPlayer> Players = new Dictionary<string, ArenaPlayer>();

            public ArenaData()
            {

            }
        }

        class Arena
        {
            public int Players;
            public bool isOpen;
            public Dictionary<string, float> Location;

            public Arena()
            {
                Players = 0;
                isOpen = false;
                Location = new Dictionary<string, float>();
            }
        }

        class ArenaPlayer
        {
            public string SteamID;
            public string CommandInterval;
            public bool isAlive;
            public float Infamy;

            public ArenaPlayer()
            {

            }

            public ArenaPlayer(PlayerSession player)
            {
                SteamID = player.SteamId.ToString();
                CommandInterval = "0";
                isAlive = false;
                Infamy = 0;
            }
        }

        #endregion

        ArenaData arenaData;
        static readonly DateTime epoch = new DateTime(2017, 1, 13, 17, 44, 0);
        static double CurrentTime() { return DateTime.UtcNow.Subtract(epoch).TotalSeconds; }

        void DoClearInventory(PlayerSession target)
        {
            var pInventory = target.WorldPlayerEntity.GetComponent<PlayerInventory>();
            for (var i = 0; i < pInventory.Capacity; i++) pInventory.Items[i] = null;

            // Add zero coal to refresh the inventory
            var itemMgr = GlobalItemManager.Instance;
            itemMgr.GiveItem(target.Player, itemMgr.GetItem(20), 0);
        }

        void OnPlayerConnected(PlayerSession player)
        {
            ArenaPlayer p = new ArenaPlayer(player);
            if (!arenaData.Players.ContainsKey(p.SteamID))
            {
                arenaData.Players.Add(p.SteamID, p);
                SaveData();
            }
        }

        void OnPlayerDeath(PlayerSession player, EntityEffectSourceData source)
        {
            if (arenaData.Players[player.SteamId.ToString()].isAlive == true)
            {
                arenaData.Arena["pvp"].Players--;
                arenaData.Players[player.SteamId.ToString()].isAlive = false;
                EntityStats stats = player.WorldPlayerEntity.GetComponent<EntityStats>();
                stats.GetFluidEffect(EEntityFluidEffectType.Infamy).SetValue(arenaData.Players[player.SteamId.ToString()].Infamy);
                arenaData.Players[player.SteamId.ToString()].Infamy = 0;
                SaveData();
                DoClearInventory(player);
                isArenaFinish("pvp");
            }
        }

        void OnPlayerDisconnected(PlayerSession player)
        {
            if (arenaData.Players[player.SteamId.ToString()].isAlive == true)
            {
                arenaData.Arena["pvp"].Players--;
                arenaData.Players[player.SteamId.ToString()].isAlive = false;
                EntityStats stats = player.WorldPlayerEntity.GetComponent<EntityStats>();
                stats.GetFluidEffect(EEntityFluidEffectType.Infamy).SetValue(arenaData.Players[player.SteamId.ToString()].Infamy);
                arenaData.Players[player.SteamId.ToString()].Infamy = 0;
                stats.GetFluidEffect(EEntityFluidEffectType.Health).SetValue(-1);
                SaveData();
                DoClearInventory(player);
                isArenaFinish("pvp");
            }
        }

        void isArenaFinish(string name)
        {
            if (arenaData.Arena[name].Players <= 1)
            {
                var winner = arenaData.Players.Values.Where(p => p.isAlive == true);
                foreach (var w in winner)
                {
                    w.isAlive = false;
                    EntityStats stats = Player.FindById(w.SteamID).WorldPlayerEntity.GetComponent<EntityStats>();
                    stats.GetFluidEffect(EEntityFluidEffectType.Infamy).SetValue(arenaData.Players[w.SteamID].Infamy);
                    arenaData.Arena["pvp"].Players = 0;
                    arenaData.Players[Player.FindById(w.SteamID).SteamId.ToString()].Infamy = 0;
                    DoClearInventory(Player.FindById(w.SteamID));
                    SaveData();
                    Server.Broadcast("<color=#4080FF>|i|</color> Zwycięzcą Areny został: " + Player.FindById(w.SteamID).Name);
                }
            }
        }

        void Loaded()
        {
            arenaData = Interface.GetMod().DataFileSystem.ReadObject<ArenaData>("ArenaData");
            if (!arenaData.Arena.ContainsKey("pvp"))
            {
                Arena a = new Arena();
                arenaData.Arena.Add("pvp", a);
                SaveData();
            }
            else
            {
                var players = arenaData.Players.Values.Where(p => p.isAlive == true);
                foreach (var user in players)
                {
                    user.isAlive = false;
                    user.Infamy = 0;
                    SaveData();
                }
                arenaData.Arena["pvp"].isOpen = false;
                SaveData();
            }
        }

        [ChatCommand("arena")]
        void arenaCmd(PlayerSession player, string command, string[] args)
        {
            if (args.Length < 1)
            {
                hurt.SendChatMessage(player, null, "<color=#4080FF>|i|</color> /arena dolacz - Dołączasz do Areny");
                hurt.SendChatMessage(player, null, "<color=#4080FF>|i|</color> /arena autor - Sprawdzasz autora Areny");
                if (player.IsAdmin)
                {
                    hurt.SendChatMessage(player, null, "<color=#4080FF>|i|</color> /arena dodaj");
                    hurt.SendChatMessage(player, null, "<color=#4080FF>|i|</color> /arena otworz");
                    hurt.SendChatMessage(player, null, "<color=#4080FF>|i|</color> /arena zamknij");
                    hurt.SendChatMessage(player, null, "<color=#4080FF>|i|</color> /arena start");
                }
            }
            else if (args[0].Equals("dolacz"))
            {
                if (args.Length < 2)
                {
                    if (isArenaExists("pvp"))
                    {
                        if (isArenaOpen("pvp"))
                        {
                            if (canPlayerJoinArena(player))
                            {
                                if (arenaData.Players[player.SteamId.ToString()].isAlive != true)
                                {
                                    player.WorldPlayerEntity.transform.position = getVector(arenaData.Arena["pvp"].Location["x"], arenaData.Arena["pvp"].Location["y"], arenaData.Arena["pvp"].Location["z"]);
                                    hurt.SendChatMessage(player, null, "<color=#4080FF>|i|</color> Dołączyłeś do Areny, poczekaj na Start!");
                                    arenaData.Arena["pvp"].Players++;
                                    arenaData.Players[player.SteamId.ToString()].isAlive = true;
                                    EntityStats stats = player.WorldPlayerEntity.GetComponent<EntityStats>();
                                    arenaData.Players[player.SteamId.ToString()].Infamy = stats.GetFluidEffect(EEntityFluidEffectType.Infamy).GetValue();
                                    SaveData();
                                }
                                else
                                {
                                    hurt.SendChatMessage(player, null, "<color=red>|b|</color> Jesteś już na Arenie!");
                                }
                            }
                            else
                            {
                                hurt.SendChatMessage(player, null, "<color=red>|b|</color> Żeby dołączyć do areny nie możesz mieć nic przy sobie!");
                                hurt.SendChatMessage(player, null, "<color=red>|b|</color> Zero przedmiotów na arenę, musisz być kompletnie pusty!");
                            }
                        }
                        else
                        {
                            hurt.SendChatMessage(player, null, "<color=red>|b|</color> Arena jest aktualnie wyłączona");
                        }
                    }
                }
            }
            else if (args[0].Equals("autor"))
            {
                if (args.Length < 2)
                {
                    hurt.SendChatMessage(player, null, "<color=red>Autor</color>: 531devv@gmail.com");
                }
            }
            else if (args[0].Equals("start"))
            {
                if (args.Length < 2)
                {
                    if (player.IsAdmin)
                    {
                        if (arenaData.Arena["pvp"].isOpen == true)
                        {
                            arenaData.Arena["pvp"].isOpen = false;
                            SaveData();
                        }

                        var players = arenaData.Players.Values.Where(p => p.isAlive == true);
                        foreach (var user in players)
                        {
                            GivePlayerItem(Player.FindById(user.SteamID), 7, 1); // siekiera
                            GivePlayerItem(Player.FindById(user.SteamID), 52, 100); // siekiera
                            EntityStats stats = Player.FindById(user.SteamID).WorldPlayerEntity.GetComponent<EntityStats>();
                            stats.GetFluidEffect(EEntityFluidEffectType.Infamy).SetValue(100f);
                            stats.GetFluidEffect(EEntityFluidEffectType.Health).SetValue(100f);
                        }
                        hurt.BroadcastChat(null, "<color=#4080FF>|i|</color> Arena wystartowała!");
                    }
                    else
                    {
                        hurt.SendChatMessage(player, null, "<color=red>|b|</color> Nie możesz użyć tej komendy!");
                    }
                }
            }
            else if (args[0].Equals("dodaj"))
            {
                if (args.Length < 2)
                {
                    if (player.IsAdmin)
                    {
                        var x = player.WorldPlayerEntity.transform.position.x;
                        var y = player.WorldPlayerEntity.transform.position.y;
                        var z = player.WorldPlayerEntity.transform.position.z;

                        if (!arenaData.Arena["pvp"].Location.ContainsKey("x"))
                        {
                            arenaData.Arena["pvp"].Location.Add("x", x);
                            arenaData.Arena["pvp"].Location.Add("y", y);
                            arenaData.Arena["pvp"].Location.Add("z", z);
                            SaveData();
                        }
                        else
                        {
                            arenaData.Arena["pvp"].Location["x"] = x;
                            arenaData.Arena["pvp"].Location["y"] = y;
                            arenaData.Arena["pvp"].Location["z"] = z;
                            SaveData();
                        }

                        hurt.SendChatMessage(player, null, "<color=#4080FF>|i|</color> Pomyślnie dodałeś Arenę!");
                        hurt.SendChatMessage(player, null, "<color=#4080FF>|i|</color> Żeby otworzyć event wpisz: /arena otworz!");
                    }
                    else
                    {
                        hurt.SendChatMessage(player, null, "<color=red>|b|</color> Nie możesz użyć tej komendy!");
                    }
                }
            }
            else if (args[0].Equals("otworz"))
            {
                if (args.Length < 2)
                {
                    if (player.IsAdmin)
                    {
                        if (isArenaExists("pvp"))
                        {
                            if (arenaData.Arena["pvp"].isOpen != true)
                            {
                                Server.Broadcast("<color=#4080FF>|i|</color> Arena na BOLTY otwarta! Schowaj itemy i dołącz do eventu: /arena dolacz");
                                arenaData.Arena["pvp"].isOpen = true;
                                SaveData();
                            }
                            else
                            {
                                hurt.SendChatMessage(player, null, "<color=#4080FF>|i|</color> Arena jest otwarta!");
                            }
                        }
                        else
                        {
                            hurt.SendChatMessage(player, null, "<color=red>|b|</color> Najpierw dodaj Arene! /arena dodaj");
                        }
                    }
                    else
                    {
                        hurt.SendChatMessage(player, null, "<color=red>|b|</color> Nie możesz użyć tej komendy!");
                    }
                }
            }
            else if (args[0].Equals("zamknij"))
            {
                if (args.Length < 2)
                {
                    if (player.IsAdmin)
                    {
                        if (isArenaExists("pvp"))
                        {
                            if (arenaData.Arena["pvp"].isOpen != false)
                            {
                                Server.Broadcast("<color=#4080FF>|i|</color> Dołączanie do Areny zamknięte!");
                                arenaData.Arena["pvp"].isOpen = false;
                                SaveData();
                            }
                            else
                            {
                                hurt.SendChatMessage(player, null, "<color=#4080FF>|i|</color> Arena jest zamknięta!");
                            }
                        }
                        else
                        {
                            hurt.SendChatMessage(player, null, "<color=red>|b|</color> Najpierw dodaj Arene! /arena dodaj");
                        }
                    }
                    else
                    {
                        hurt.SendChatMessage(player, null, "<color=red>|b|</color> Nie możesz użyć tej komendy!");
                    }
                }
            }
        }

        #region API

        private string isPlayerOnArena(PlayerSession player)
        {
            if (arenaData.Players[player.SteamId.ToString()].isAlive == true)
            {
                return "true";
            }
            return "false";
        }

        #endregion

        #region Helpers

        bool canPlayerJoinArena(PlayerSession player)
        {
            var playerinv = player.WorldPlayerEntity.GetComponent<PlayerInventory>();
            List<string> items = new List<string>();
            int count = 0;
            for (int i = 0; i < playerinv.Capacity; i++)
            {
                if (playerinv.Items[i] != null)
                {
                    count++;
                }
            }
            if (count < 1)
            {
                return true;
            }
            return false;
        }

        bool isArenaOpen(string name)
        {
            if (arenaData.Arena[name].isOpen == true)
            {
                return true;
            }
            return false;
        }

        bool isArenaExists(string name)
        {
            if (arenaData.Arena.ContainsKey(name))
            {
                return true;
            }
            return false;
        }

        private static bool isValid(String str)
        {
            return Regex.IsMatch(str, @"^[a-zA-Z]+$");
        }

        bool isCommandIntervalTimeOut(PlayerSession player)
        {
            ArenaPlayer p = new ArenaPlayer(player);
            string interval = arenaData.Players[p.SteamID].CommandInterval;
            var finish = Convert.ToDouble(interval) - CurrentTime();
            if (finish <= 0)
            {
                return true;
            }
            return false;
        }

        void setCommandInterval(PlayerSession player)
        {
            ArenaPlayer p = new ArenaPlayer(player);
            arenaData.Players[p.SteamID].CommandInterval = (CurrentTime() + 3).ToString();
            SaveData();
        }

        PlayerSession GetPlayer(string searchedPlayer, PlayerSession player)
        {
            foreach (PlayerSession current in GameManager.Instance.GetSessions().Values)
                if (current != null && current.Name != null && current.IsLoaded && current.Name.ToLower() == searchedPlayer)
                    return current;

            List<PlayerSession> foundPlayers =
                (from current in GameManager.Instance.GetSessions().Values
                 where current != null && current.Name != null && current.IsLoaded && current.Name.ToLower().Contains(searchedPlayer.ToLower())
                 select current).ToList();

            switch (foundPlayers.Count)
            {
                case 0:
                    hurt.SendChatMessage(player, null, "Nie ma takiego gracza!");
                    break;

                case 1:
                    return foundPlayers[0];

                default:
                    List<string> playerNames = (from current in foundPlayers select current.Name).ToList();
                    string players = ListToString(playerNames, 0, ", ");
                    hurt.SendChatMessage(player, null, "Znaleziono kilku graczy: \n" + players);
                    break;
            }

            return null;
        }

        public Vector3 getVector(float x, float y, float z)
        {
            return new Vector3(x, y, z);
        }

        public void SaveData() => Interface.GetMod().DataFileSystem.WriteObject("ArenaData", arenaData);

        public List<OwnershipStakeServer> GetStakesFromPlayer(PlayerSession session)
        {
            var stakes = Resources.FindObjectsOfTypeAll<OwnershipStakeServer>();
            if (stakes != null)
            {
                return
                    stakes.Where(
                        s =>
                            !s.IsDestroying && s.gameObject != null && s.gameObject.activeSelf &&
                            s.AuthorizedPlayers.Contains(session.Identity)).ToList();
            }
            return new List<OwnershipStakeServer>();
        }

        public PlayerSession GetPlayerSessionFromDataSource(EntityEffectSourceData dataSource)
        {
            return GameManager.Instance.GetSession(dataSource.EntitySource.GetComponent<uLinkNetworkView>().owner);
        }

        void GivePlayerItem(PlayerSession player, int itemID, int amount)
        {
            var ItemMgr = Singleton<GlobalItemManager>.Instance;
            ItemMgr.GiveItem(player.Player, ItemMgr.GetItem(itemID), amount);
        }

        string ListToString(List<string> list, int first, string seperator) => string.Join(seperator, list.Skip(first).ToArray());

        #endregion
    }
}
